@@ -1,4 +1,4 @@
-# Copyrights 2006-2016 by [Mark Overmeer].
+# Copyrights 2006-2017 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 2.02.
@@ -8,7 +8,7 @@ no warnings 'recursion';
 
 package XML::Compile::Schema::BuiltInTypes;
 use vars '$VERSION';
-$VERSION = '1.52';
+$VERSION = '1.58';
 
 use base 'Exporter';
 
@@ -21,9 +21,11 @@ use POSIX           qw/strftime/;
 use Math::BigInt;
 use Math::BigFloat;
 use MIME::Base64;
+use Types::Serialiser;
+use Scalar::Util    qw(dualvar);
+use POSIX           qw/floor log10/;
 
 use XML::Compile::Util qw/pack_type unpack_type/;
-use POSIX              qw/floor log10/;
 
 use Config '%Config';
 my $iv_bits   = $Config{ivsize} * 8 -1;
@@ -49,7 +51,7 @@ sub identity  { $_[0] }
 sub str2int   { $_[0] + 0 }
 
 # sprintf returns '0' if non-int, with warning. We need a validation error
-sub int2str   { $_[0] =~ m/^\s*[0-9]+\s*$/ ? sprintf("%ld", $_[0]) : $_[0] }
+sub int2str   { $_[0] =~ m/^\s*([0-9]+)\s*$/ ? $1 : $_[0] }
 
 sub str       { "$_[0]" }
 sub _replace  { $_[0] =~ s/[\t\r\n]/ /g; $_[0]}
@@ -78,6 +80,8 @@ $builtin_types{anyAtomicType} =
 
 $builtin_types{error}   = {example => '[some error structure]'};
 
+#----------------
+
 
 $builtin_types{boolean} =
  { parse   => sub { $_[0] =~ m/^\s*false|0\s*/i ? 0 : 1 }
@@ -88,6 +92,16 @@ $builtin_types{boolean} =
  , extends => 'anyAtomicType'
  };
 
+$builtin_types{boolean_with_Types_Serialiser} =
+ { %{$builtin_types{boolean}}
+ , parse => sub {
+       no warnings 'once';
+       $_[0] =~ m/^\s*(false|0)\s*/i
+       ? $Types::Serialiser::false
+       : $Types::Serialiser::true;
+    }
+ };
+
 
 $builtin_types{pattern} =
  { example => '*.exe'
@@ -97,7 +111,11 @@ $builtin_types{pattern} =
 sub bigint
 {   my $v = shift;
     $v =~ s/\s+//g;
-    return $v if $v =~ $fits_iv;
+
+	# The automatic rewrite into JSON wants real ints, not strings.  Therefore,
+	# we need to numify.  On the other hand, pattern matching/enumeration
+	# requires the original string.  Regression tests prove this trick works.
+    return dualvar($v+0, $v) if $v =~ $fits_iv;
 
     my $big = Math::BigInt->new($v);
     error __x"Value `{val}' is not a (big) integer", val => $big
@@ -250,7 +268,7 @@ $builtin_types{unsignedByte} =
 
 
 $builtin_types{decimal} =
- { parse   => sub {$_[0] =~ s/\s+//g; Math::BigFloat->new($_[0]) }
+ { parse   => sub {$_[0] =~ s/\s+//g; Math::BigFloat->new($_[0]) },
  , check   => sub {$_[0] =~ m/^(\+|\-)?([0-9]+(\.[0-9]*)?|\.[0-9]+)$/}
  , example => '3.1415'
  , extends => 'anyAtomicType'
@@ -262,7 +280,7 @@ sub str2num
     $s =~ s/\s//g;
 
       $s =~ m/[^0-9]/ ? Math::BigFloat->new($s eq 'NaN' ? $s : lc $s) # INF->inf
-    : length $s < 9   ? $s+0
+    : length $s < 9   ? dualvar($s+0, $s)
     :                   Math::BigInt->new($s);
 }
 
@@ -282,7 +300,7 @@ sub numcheck($)
        | NaN #x
 }
 
-$builtin_types{precissionDecimal} =
+$builtin_types{precisionDecimal} =
 $builtin_types{float}  =
 $builtin_types{double} =
  { parse   => \&str2num
@@ -302,9 +320,14 @@ $builtin_types{sloppy_float} =
  , extends => 'anyAtomicType'
  };
 
+$builtin_types{sloppy_float_force_NV} =
+ { %{$builtin_types{sloppy_float}}
+ , parse => sub { $_[0] + 0 }
+ };
+
 
 $builtin_types{base64Binary} =
- { parse   => sub { eval { decode_base64 $_[0] } }
+ { parse   => sub { eval { decode_base64 $_[0] }; }
  , format  => sub {
        my $a = $_[0];
        eval { utf8::downgrade($a) };
@@ -322,10 +345,10 @@ $builtin_types{base64Binary} =
 
 # (Use of) an XS implementation would be nice
 $builtin_types{hexBinary} =
- { parse   => sub { $_[0] =~ s/\s+//g; pack 'H*', $_[0]}
+ { parse   => sub { (my $v = $_[0]) =~ s/\s+//g; pack 'H*', $v }
  , format  => sub { uc unpack 'H*', $_[0]}
- , check   =>
-     sub { $_[0] !~ m/[^0-9a-fA-F\s]/ && (($_[0] =~ tr/0-9a-fA-F//) %2)==0}
+ , check   => sub { (my $v = $_[0]) !~ m/[^0-9a-fA-F\s]/ or return 0;
+     ($v =~ tr/0-9a-fA-F//) % 2 == 0}
  , example => 'F00F'
  , extends => 'anyAtomicType'
  };
@@ -560,6 +583,7 @@ $builtin_types{NMTOKEN} =
 
 $builtin_types{NMTOKENS} =
  { parse   => sub { [ split ' ', shift ] }
+ , check   => sub { $_[0] =~ /\S/ }
  , format  => sub { my $v = shift; ref $v eq 'ARRAY' ? join(' ',@$v) : $v }
  , example => 'nmtokens'
  , is_list => 1
